#DEFINE G_LOG_DOMAIN "ap-sheet"

#include "sheet.h"

#include <glib.h>

#include <libxml/xmlreader.h>
#include <libxml/xmlwriter.h>

//TODO: Needed?
#include <libxml/tree.h>
#include <libxml/parser.h>

typedef struct _APSheetXml {
	APGameXml game;
	APBioXml bio;
	APStatsXml stats;
	APAttacksXml attacks;
	APInventoryXml inventory;
} APSheetXml;

typedef struct _APSheetPrivate
{
    APSheetXml *sheet;
} APSheetPrivate;

struct _APSheet
{
    GObject parent_instance;
};

G_DEFINE_TYPE_WITH_PRIVATE (APSheet, ap_sheet, GObject)

enum
{
    PROP_FILE,
    N_PROPERTIES
};

static GParamSpec *obj_properties[N_PROPERTIES] = { NULL, };

static void
ap_sheet_set_property (GObject      *object,
                       guint         property_id,
                       const GValue *value,
                       GParamSpec   *pspec)
{
    APSheet *self = AP_SHEET (object);

    switch (property_id)
    {
        case PROP_FILE:
            //self->priv->filename = g_value_dup_string (value);
            break;
        
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

static void
ap_sheet_get_property (GObject      *object,
                       guint         property_id,
                       const GValue *value,
                       GParamSpec   *pspec)
{
    APSheet *self = AP_SHEET (object);

    switch (property_id)
    {
        case PROP_FILE:
            //g_value_set_string (value, self->priv->filename);
            break;

        default
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

/*
 * boilerplate_window_constructed:
 * @object: An instance of #BoilerplateWindow
 *
 * The constructed virtual function sets first-run settings.
 */
static void
ap_sheet_constructed (GObject *object)
{
    APSheet *self = AP_SHEET (object);
    APSheetPrivate *priv = ap_sheet_get_instance_private (self);

    G_OBJECT_CLASS (ap_sheet_parent_class)->constructed (object);
}

/*
 * ap_sheet_finalize:
 * @object: An instance of #APSheet
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
ap_sheet_finalize (GObject *object)
{
    APSheetPrivate *priv = ap_sheet_get_instance_private (AP_SHEET (object));

    G_OBJECT_CLASS (ap_sheet_parent_class)->finalize (object);
}

/*
 * ap_sheet_init:
 * @self: An instance of #APSheet
 *
 * Initializes various aspects of the window. Anything the window would need
 * before becoming presented should be done here.
 */
static void
ap_sheet_init (APSheet *self)
{
    APSheetPrivate *priv = ap_sheet_get_instance_private (self);
}

/*
 * ap_sheet_class_init:
 * @klass: An instance of #APSheetClass
 *
 * Initializes the various virtural functions in #APSheetClass.
 */
static void
ap_sheet_class_init (APSheetClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->constructed = ap_sheet_constructed;
    object_class->finalize = ap_sheet_finalize;
    
    object_class->set_property = ap_sheet_set_property;
    object_class->get_property = ap_sheet_get_property;
    
    obj_properties[PROP_FILE] =
        g_param_spec_string ("file",
                             "file",
                             "Name of the file to load and display from.",
                             NULL  /* default value */,
                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE));

    g_object_class_install_properties (object_class,
                                       N_PROPERTIES,
                                       obj_properties);
}

/**
 * ap_sheet_new:
 * 
 * Creates a new #APSheet, a sub-object of #GtkWindow.
 *
 * ## Usage
 * |[
 * APSheet *sheet;
 *
 * sheet = ap_sheet_new ();
 * ]|
 *
 * Returns: #APSheet; a sub-object of #GObject
 */
APSheet *
ap_sheet_new (void)
{
    return g_object_new (AP_TYPE_SHEET,
                         NULL);
}

/**
 * ap_sheet_new:
 * 
 * Creates a new #APSheet, a sub-object of #GtkWindow.
 *
 * ## Usage
 * |[
 * APSheet *sheet;
 *
 * sheet = ap_sheet_from_file ("./my-sheet.xml");
 * ]|
 *
 * Returns: #APSheet; a sub-object of #GObject
 */
APSheet *
ap_sheet_from_file (const gchar *file)
{
    return g_object_new (AP_TYPE_SHEET,
						 "file", file,
                         NULL);
}


